/*
    checkUUID - Check a Universally Unique Identifier (UUID)
    Copyright  2007-2016 Harry Whitfield

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    checkUUID - version 1.8
    15 February, 2016
    Copyright  2007-2016 Harry Whitfield
    mailto:g6auc@arrl.net

    This function is for use in Web Widgets - it is based on ITU-T X.667 - UUID Version 1
*/

/*jslint browser, multivar */

/*property
    UTC, floor, match, substring
*/

function dateOf(uuid) { // 651224f0-eb5f-11db-87c3-0016cb01fc35
                        // 012345678901234567890
    "use strict";
    var xTimeHigh = uuid.substring(15, 18),
        xTimeMid = uuid.substring(9, 13),
        xTimeLow = uuid.substring(0, 8),
        mS = Math.floor(Number("0x" + xTimeHigh + xTimeMid + xTimeLow.substring(0, 7)) / 625);

    return new Date(Date.UTC(1582, 9, 15) + mS);
}

function checkUUID(uuid, lowVariant, highVariant) { // 651224f0-eb5f-11db-87c3-0016cb01fc35
    "use strict";
    var xVersion, xVariant, xVarCkSeq,
            version = null,
            variant,
            lookFor = /^([A-F0-9]{8})-([A-F0-9]{4})-([A-F0-9]{4})-([A-F0-9]{4})-([A-F0-9]{12})$/i,
            found = uuid.match(lookFor);

    if (found !== null) {
        xVersion = found[3].substring(0, 1);
        xVarCkSeq = found[4];

        version = parseInt(xVersion, 16);
        if ((version < 1) || (version > 5)) {
            version = null;
        }

        xVariant = xVarCkSeq.substring(0, 1);
        variant = parseInt(xVariant, 16);

        if ((variant < lowVariant) || (variant > highVariant)) {
            version = null;
        }
        version = [version, variant];
    }
    return version;
}
