/*
    UUID Decode - Displays Properties of UUIDs
    Copyright 2014,2015 Harry Whitfield

    This program is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License as published by the Free Software
    Foundation; either version 2 of the License, or (at your option) any later
    version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
    PARTICULAR PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
    St, Fifth Floor, Boston, MA  02110-1301  USA

    UUID Decode - browser version 1.0
    3 May, 2015
    Copyright 2014,2015 Harry Whitfield
    mailto:g6auc@arrl.net
 */

/*jslint browser, multivar */

/*global paramOne, paramTwo, paramThree, paramFour, clearData, Menu, dateOf, checkUUID
*/

/*properties
    getTime, join, keyCode, onchange, onkeypress, selectedIndex, split, toString,
    toUTCString, value, which
*/

// map custom names onto model names
var uuidText = paramOne;
var timeText = paramTwo;
var uuidVersion = paramThree;
var messageText = paramFour;

/*
var referenceHeading = headingOne;
var latitudeHeading = headingTwo;
var longitudeHeading = headingThree;
var locatorHeading = headingFour;
*/

var variantPrefMenu = Menu[0];
var utcFormatPrefMenu = Menu[1];
var timeFormatPrefMenu = Menu[2];

/*
    ['X.667 only', 'X.667 and NCS', 'X.667 and Microsoft', 'X.667, NCS and Microsoft'],
    ['Local Time', 'Use Universal Time'],
    ['12-hour', '24-hour']
*/

var variantPref = 0;    // 'X.667 only'
var utcFormatPref = "Use Universal Time";
var timeFormatPref = "24-hour";

var utc = (utcFormatPref === "Use Universal Time");    // local/utc mode
var mode12 = (timeFormatPref === "12-hour");            // 12/24 hour mode

function time12HourOf(time) { // time in format hh:mm:ss
    "use strict";
    var ampm = "am",
        item = time.split(":"),
        hours = parseInt(item[0], 10);

    if (hours > 12) {
        hours -= 12;
        ampm = "pm";
    } else if (hours === 12) {
        ampm = "pm";
    } else if (hours === 0) {
        hours = 12;
    }
    item[0] = String(hours);
    if (hours < 10) {
        item[0] = " " + item[0];
    }
    return item.join(":") + " " + ampm;
}

function displayDate(theDate) {
    "use strict";
    var item, theString;

    if (utc) {
        theString = theDate.toUTCString();
    } else {
        theString = theDate.toString();
    }
    if (mode12) {
        item = theString.split(" ");
        item[4] = time12HourOf(item[4]);
        theString = item.join(" ");
    }
    return theString;
}

function creationTime(version, uuid) {
    "use strict";
    var theDate, mS, then, now,
            tolerance = 1800000;    // 30 minutes to allow for badly set clocks.

    if (version === 0) {
        return "The UUID is invalid.";
    }
    if (version !== 1) {
        return "The UUID has no creation time.";
    }
    theDate = dateOf(uuid);
    mS = theDate.getTime();
    then = new Date(1970, 0, 1).getTime();
    now = new Date().getTime();
    if ((mS < then) || (mS > now + tolerance)) {
        return "The timestamp is invalid.";
    }
    return displayDate(theDate) + " +" + (mS % 1000) + "mS";
}

function process(text) {
    "use strict";
    var version,
        variant,
        lowVariant,
        highVariant;

    function nameOf(variant) {
        if (variant < 8) {
            return "NCS";
        }
        if (variant < 12) {
            return "X.667";
        }
        if (variant < 14) {
            return "MSoft"; // Microsoft
        }
        return "Reserved";
    }

    switch (variantPref) {
    case 1:
        lowVariant = 0;
        highVariant = 11;
        break; // X.667 and NCS
    case 2:
        lowVariant = 8;
        highVariant = 13;
        break; // X.667 and Microsoft
    case 3:
        lowVariant = 0;
        highVariant = 13;
        break; // X.667, NCS and Microsoft
    default:
        lowVariant = 8;
        highVariant = 11;
        break; // X.667 only
    }

    version = checkUUID(text, lowVariant, highVariant);

    if (version !== null) {
        variant = version[1];
        version = version[0];
        timeText.value = creationTime(version, text);
        uuidVersion.value = version + " (" + variant + ":" + nameOf(variant) + ")";
    } else {
        timeText.value = "";
        uuidVersion.value = "";
        messageText.value = "The UUID is invalid.";
    }
}

variantPrefMenu.onchange = function () {
    "use strict";
    variantPref = variantPrefMenu.selectedIndex;
    process(uuidText.value);
};

utcFormatPrefMenu.onchange = function () {
    "use strict";
    utcFormatPref = utcFormatPrefMenu.value;
    utc = (utcFormatPref === "Use Universal Time");
    process(uuidText.value);
};

timeFormatPrefMenu.onchange = function () {
    "use strict";
    timeFormatPref = timeFormatPrefMenu.value;
    mode12 = (timeFormatPref === "12-hour");
    process(uuidText.value);
};

uuidText.onkeypress = function (event) {
    "use strict";
    var x = event.which || event.keyCode;

    if (x === 13) {
        process(uuidText.value);
    } else {
        timeText.value = "";
        uuidVersion.value = "";
        messageText.value = "";
    }
};

//updatePrefs();
