/*
    Web Widget
    Copyright 2014,2015 Harry Whitfield

    This program is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License as published by the Free Software
    Foundation; either version 2 of the License, or (at your option) any later
    version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
    PARTICULAR PURPOSE.  See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
    St, Fifth Floor, Boston, MA  02110-1301  USA

    Web Widget - browser version 1.0.4
    4 July, 2015
    Copyright 2014,2015 Harry Whitfield
    mailto:g6auc@arrl.net
 */

/*jslint browser, multivar, this */

/*globals alert, document */

/*property
    PI, add, appendChild, body, clearRect, cols, createElement, createTextNode,
    ctx, div, drawImage, fillText, font, forEach, getContext, hRegP, height,
    left, onmousedown, onmouseenter, onmouseout, onmouseup, opacity, restore,
    rotate, round, rows, save, scale, setAttribute, setStyle, size, src,
    srcDown, srcUp, style, text, top, translate, type, vRegP, value, width,
    within, zOrder
*/

//////////////////////////// Start of the webWidget functions ////////////////////////////

var scale = 1.0, vtop = 0, left = 0, fScale = 6.0;    // was 6.67

function newElement(tagName, hOffset, vOffset, zOrder) {
    "use strict";
    var div = document.createElement("div"), ele;

    div.setAttribute("style", "position: absolute; left: " + String(hOffset + left) +
            "px; top: " + String(vOffset + vtop) + "px; z-index: " + String(zOrder) + ";");

    document.body.appendChild(div);

    ele = document.createElement(tagName);
    ele.left = hOffset;
    ele.top = vOffset;
    ele.div = div;
    ele.zOrder = zOrder;
    div.appendChild(ele);

    return ele;
}

function moveObj(obj, hOffset, vOffset) {
    "use strict";
    hOffset = Math.round(scale * hOffset);
    vOffset = Math.round(scale * vOffset);

    obj.div.setAttribute("style", "position: absolute; left: " + String(hOffset + left) +
            "px; top: " + String(vOffset + vtop) + "px; z-index: " + String(obj.zOrder) + ";");
}

function move(obj) {
    "use strict";
    obj.div.setAttribute("style", "position: absolute; left: " + String(obj.left + left) +
            "px; top: " + String(obj.top + vtop) + "px; z-index: " + String(obj.zOrder) + ";");
}

function newImage(hOffset, vOffset, width, height, src, zOrder, opacity) {
    "use strict";
    var img;

    hOffset = Math.round(scale * hOffset);
    vOffset = Math.round(scale * vOffset);

    img = newElement("img", hOffset, vOffset, zOrder);

    img.width = Math.round(scale * width);
    img.height = Math.round(scale * height);
    img.src = src;
    if (opacity === undefined) {
        opacity = 1.0;
    }   // opacity is an optional parameter
    img.style.opacity = opacity;
    return img;
}

function newButton(hOffset, vOffset, width, height, src, srcDown, zOrder, opacity, clickAction) {
    "use strict";
	var o = newImage(hOffset, vOffset, width, height, src, zOrder, opacity);

	o.srcUp = src;
	o.srcDown = srcDown;
	o.within = false;
	o.onmouseup = function () {
		if (this.within) {
			this.src = src;
			clickAction();
		}
	};
	o.onmousedown = function () {
		this.within = true;
		this.src = srcDown;
	};
	o.onmouseenter = function () {
		this.within = true;
		this.src = srcDown;
	};
	o.onmouseout = function () {
		this.within = false;
		this.src = src;
	};
	return o;
}

function newText(hOffset, vOffset, width, height, value, zOrder, style) {
    "use strict";
    var text;

    hOffset = Math.round(scale * hOffset);
    vOffset = Math.round(scale * vOffset);

    text = newElement("input", hOffset, vOffset, zOrder);
    text.type = "text";

    text.width = Math.round(scale * width);
    text.size = Math.round(scale * width / fScale);
    text.height = Math.round(scale * height);
    text.value = value;
    if (style !== undefined) {  // style is an optional parameter
        text.style.setStyle(style);
    }
    return text;
}

function newPara(hOffset, vOffset, ignore, height, value, zOrder, style) {
    "use strict";
    var para, text;

    vOffset = vOffset - 9 * height / 16;

    hOffset = Math.round(scale * hOffset);
    vOffset = Math.round(scale * vOffset);

    para = newElement("p", hOffset, vOffset, zOrder);
    text = document.createTextNode(value);
    if (style !== undefined) {  // style is an optional parameter
        para.style.setStyle(style);
    }
    para.appendChild(text);
    return para;                // para.innerHTML = value to change the text
}

function newTextArea(hOffset, vOffset, cols, rows, value, zOrder, style) {
    "use strict";
    var textarea;

    hOffset = Math.round(scale * hOffset);
    vOffset = Math.round(scale * vOffset);

    textarea = newElement("textarea", hOffset, vOffset, zOrder);

    textarea.cols = cols;
    textarea.rows = rows;
    textarea.value = value;
    if (style !== undefined) {  // style is an optional parameter
        textarea.style.setStyle(style);
    }
    return textarea;
}

function newInput(hOffset, vOffset, width, height, value, zOrder, style) {
    "use strict";
    var input;

    hOffset = Math.round(scale * hOffset);
    vOffset = Math.round(scale * vOffset);

    input = newElement("input", hOffset, vOffset, zOrder);
    input.type = "text";

    input.width = Math.round(scale * width);
    input.size = Math.round(scale * width / fScale);
    input.height = Math.round(scale * height);
    input.value = value;
    if (style !== undefined) {  // style is an optional parameter
        input.style.setStyle(style);
    }
    return input;
}

function newCanvas(hOffset, vOffset, width, height, src, zOrder, opacity, hRegP, vRegP) {
    "use strict";
    var canvas;

    hOffset = Math.round(scale * hOffset);
    vOffset = Math.round(scale * vOffset);

    canvas = newElement("canvas", hOffset, vOffset, zOrder);

    hRegP = hRegP || 0;     // hRegP and vRegP are optional parameters
    vRegP = vRegP || 0;

    canvas.src = src;
    if (opacity === undefined) {
        opacity = 1.0;
    }   // opacity is an optional parameter
    canvas.style.opacity = opacity;

    canvas.width = Math.round(scale * width);
    canvas.height = Math.round(scale * height);
    canvas.ctx = canvas.getContext("2d");
    canvas.hRegP = Math.round(scale * hRegP);
    canvas.vRegP = Math.round(scale * vRegP);
    return canvas;
}

function newCanvasImage(canvas) {
    "use strict";
    var img = new Image();

    img.width = canvas.width;
    img.height = canvas.height;
    img.src = canvas.src;
    return img;
}

function drawImage(canvas, img, hOff, vOff, ang) {
    "use strict";
    var ctx = canvas.ctx, width = canvas.width, height = canvas.height,
            hRegP = canvas.hRegP, vRegP = canvas.vRegP;

    ctx.save();
    ctx.clearRect(0, 0, width, height);
    ctx.translate(hRegP, vRegP);
    ctx.rotate(Math.PI * ang / 180);
    ctx.translate(-hRegP, -vRegP);
    ctx.scale(scale, scale);
    ctx.drawImage(img, hOff, vOff);
    ctx.restore();
}

function drawText(canvas, text, hOff, vOff, ang, font) {
    "use strict";
    var ctx = canvas.ctx, width = canvas.width, height = canvas.height,
            hRegP = canvas.hRegP, vRegP = canvas.vRegP;

    ctx.save();
    ctx.clearRect(0, 0, width, height);
    ctx.translate(hRegP, vRegP);
    ctx.rotate(Math.PI * ang / 180);
    ctx.translate(-hRegP, -vRegP);
    ctx.scale(scale, scale);
    ctx.font = font;
    ctx.fillText(text, hOff, vOff);
    ctx.restore();
}

function newSelector(hOffset, vOffset, width, height, src, zOrder, opacity) {
    "use strict";
    var selector;

    hOffset = Math.round(scale * hOffset);
    vOffset = Math.round(scale * vOffset);

    selector = newElement("select", hOffset, vOffset, zOrder);

    selector.width = Math.round(scale * width);
    selector.height = Math.round(scale * height);
    selector.src = src;
    if (opacity === undefined) {
        opacity = 1.0;
    }   // opacity is an optional parameter
    selector.style.opacity = opacity;

//  selector.style.fontWeight = "bold";
//  selector.style.fontSize = "14px";

    return selector;
}

function addToMenu(selector, strings) {
    "use strict";
    var option;

    strings.forEach(function (ele) {
        option = document.createElement("option");
        option.text = ele;
        selector.add(option);
    });
}

function newFileSelector(hOffset, vOffset, width, height, ignore, zOrder, style) {
    "use strict";
    var input;

    hOffset = Math.round(scale * hOffset);
    vOffset = Math.round(scale * vOffset);

    input = newElement("input", hOffset, vOffset, zOrder);
    input.type = "file";

    input.width = Math.round(scale * width);
    input.size = Math.round(scale * width / fScale);
    input.height = Math.round(scale * height);
//  input.value = value;
    if (style !== undefined) {  // style is an optional parameter
        input.style.setStyle(style);
    }
    return input;
}

var moveImg = moveObj;  // for backwards compatibility

///////////////////////////// End of the webWidget functions /////////////////////////////
